@@ -1,5 +1,5 @@
 /* This file is part of GNU cflow
-   Copyright (C) 1997-2019 Sergey Poznyakoff
+   Copyright (C) 1997-2024 Sergey Poznyakoff
  
    GNU cflow is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -364,12 +364,12 @@ tree_output()
 	       for (p = linked_list_head(symbols[i]->callee); p;
 		    p = p->next) {
 		    Symbol *s = (Symbol*) p->data;
-		    if (symbol_is_function(s))
-			 depmap_set(depmap, i, ((Symbol*)p->data)->ord);
+		    if (s->ord != -1 && symbol_is_function(s))
+			 depmap_set(depmap, i, s->ord);
 	       }		    
 	  }
      }
-     
+
      depmap_tc(depmap);
 
      /* Mark recursive calls */
@@ -392,16 +392,21 @@ tree_output()
 	       separator();
 	  }
      } else {
-	  if ((main_sym = start_name ? lookup(start_name) : NULL) != NULL) {
-	       direct_tree(0, 0, main_sym);
-	       separator();
+	  void *state;
+	  
+	  main_sym = first_starter(&state);
+	  if (main_sym) {
+	       do {
+		   direct_tree(0, 0, main_sym);
+		   separator();
+	       } while ((main_sym = next_starter(&state)) != NULL);
 	  } else if (!all_functions) {
 	       all_functions = 1;
 	  }
 
 	  if (all_functions) {
 	       for (i = 0; i < num; i++) {
-		    if (main_sym != symbols[i]
+		    if (symbols[i]->flag != symbol_start
 			&& symbols[i]->source
 			&& (all_functions > 1 || symbols[i]->caller == NULL)) {
 			 direct_tree(0, 0, symbols[i]);
@@ -432,6 +437,7 @@ output()
 	  xref_output();
      }
      if (print_option & PRINT_TREE) {
+	  eliminate_non_targets();
 	  tree_output();
      }
      fclose(outfile);
